Package: SPOT2
License: GPL (>=2)
Title: Sequential Parameter Optimization Toolbox, Second Release
Type: Package
LazyLoad: yes
LazyData: true
Authors@R: c(	
	      person("Thomas", "Bartz-Beielstein", role = "aut"),
				person("Joerg", "Stork", role = "aut"),
				person("Martin", "Zaefferer", role = c("aut", "cre"),email = "martin.zaefferer@gmx.de"),
	      person("Margarita", "Rebolledo", role = "ctb"),
	      person("Christian", "Lasarczyk", role = "ctb"))
Description: Provides a set of tools for model based optimization and tuning of algorithms. It includes surrogate models, optimizers and design of experiment approaches. The main interface is spot2, which uses sequentially updated surrogate models for the purpose of efficient optimization. The main goal is to ease the burden of objective function evaluations, when a single evaluation requires a significant amount of resources.
Version: 0.1
URL: http://www.spotseven.de/
Date: 2016-10-02
Depends: R (>= 3.0.0)
Imports: randomForest, stats, utils, graphics, MASS, party
RoxygenNote: 5.0.1
Suggests: testthat
NeedsCompilation: no
Packaged: 2016-11-23 19:31:00 UTC; Martin
Author: Thomas Bartz-Beielstein [aut],
  Joerg Stork [aut],
  Martin Zaefferer [aut, cre],
  Margarita Rebolledo [ctb],
  Christian Lasarczyk [ctb]
Maintainer: Martin Zaefferer <martin.zaefferer@gmx.de>
