test_that("check that spot2 function works without problems", {
  res <- spot2(,funSphere,c(-2,-3),c(1,2))
  res <- spot2(,funSphere,c(-2,-3),c(1,2),control=list(model=buildRandomForest))	
  res <- spot2(,funSphere,c(-2,-3),c(1,2),control=list(model=buildLM))	
  res <- spot2(,funSphere,c(-2,-3),c(1,2),control=list(funEvals=11,model=buildKrigingDACE))	
  res <- spot2(,funSphere,c(-2,-3),c(1,2),control=list(optimizer=optimLBFGSB))	
  res <- spot2(,funSphere,c(-2,-3),c(1,2),control=list(design=designUniformRandom))	
	res <- spot2(,funSphere,c(-2,-3),c(1,2),control=list(modelControl=list(target="ei")))
})