
##
## Copyright (C) 2015-2016 T. Bartz-Beielstein, M. Zaefferer, J. Stork, ... #TODO
## This program is free software;
## you can redistribute it and/or modify it under the terms of the 
## GNU General Public License as published by the Free Software Foundation; 
## either version 3 of the License,
## or (at your option) any later version.
## This program is distributed in the hope that it will be useful, 
## but WITHOUT ANY WARRANTY; without even the implied warranty of 
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
## See the GNU General Public License for more details.
## You should have received a copy of the GNU General Public License along
##  with this program; if not, see <http://www.gnu.org/licenses/>.
##

# Package Description for Roxygen:
#' Sequential Parameter Optimization Toolbox, Second Release
#'
#' SPOT2 uses a combination statistic models and optimization algorithms 
#' for the purpose of parameter optimization. Design of Experiment methods
#' are employed to generate an initial set of candidate solutions, which
#' are evaluated with a user-provided target function.
#' The resulting data is used to fit a model, which in turn is subject
#' to an optimization algorithm, to find the most promising next solution(s).
#' These are again evaluated, after which the model is updated with the new results.
#' This sequential procedure of modeling, optimization and evaluation is iterated
#' until the evaluation budget is exhausted. 
#'
#' Note, that an older R-package implementing the described approaches exists: SPOT.
#' The new SPOT2 package aims to improve the following issues of the older package:\cr
#' - A more modular architecture is provided, that allows the user to easily customize parts of the SPO procedure.\cr
#' - Core functions for modeling and optimization use interfaces more similar to algorithms from other packages / core-R, 
#'   hence making them easier accessible for new users. Also, these can now be more easily used separately from the main SPO 
#'   approach, e.g., only for modeling.\cr
#' - Reducing the unnecessarily large number of choices and parameters.\cr
#' - Removal of extremely rarely used / un-used features, to reduce overall complexity of the code.\cr
#' - Improving documentation and accessibility in general.\cr
#' - Speed-up of frequently used procedures.\cr
#' As this is the initial re-release version, we would appreciate feedback about any bugs or other issues with the package. 
#' Feel free to send feedback by mail to the maintainer.
#'
#' \tabular{ll}{
#' Package: \tab SPOT2\cr
#' Type: \tab Package\cr
#' Version: \tab 0.1\cr
#' Date: \tab 2016-10-02\cr
#' License: \tab GPL (>= 2)\cr
#' LazyLoad: \tab yes\cr
#' }
#'
#' @name SPOT2-package
#' @aliases SPOT2
#' @docType package
#' @title Sequential Parameter Optimization Toolbox, Second Release
#' @author Thomas Bartz-Beielstein \email{thomas.bartz-beielstein@@th-koeln.de}, Joerg Stork, Martin Zaefferer (Maintainer, \email{martin.zaefferer@@gmx.de})
#' with contributions from: C. Lasarczyk, M. Rebolledo
#' @keywords package
#' @seealso Main interface function is \code{\link{spot2}}.
#' @import randomForest
#' @import stats
#' @import MASS
#' @import utils
#' @import graphics
#' 
#' @section Acknowledgments: 
#' This work has been partially supported by the Federal Ministry of Education
#' and Research (BMBF) under the grants CIMO (FKZ 17002X11) and
#' MCIOP (FKZ 17N0311).
#'
#' @section Maintainer:
#' Martin Zaefferer \email{martin.zaefferer@@gmx.de} 
#End of Package Description
NA 