% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/buildRandomForest.R
\name{predict.spot2RandomForest}
\alias{predict.spot2RandomForest}
\title{Prediction method for random forest}
\usage{
\method{predict}{spot2RandomForest}(object, newdata, ...)
}
\arguments{
\item{object}{fit of the model, an object of class \code{"spot2RandomForest"}, produced by \code{\link{buildRandomForest}}.}

\item{newdata}{matrix of new data.}

\item{...}{not used}
}
\description{
Wrapper to \code{predict.randomForest}.
}
\keyword{internal}

