% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/buildRandomForest.R
\name{print.spot2RandomForest}
\alias{print.spot2RandomForest}
\title{Print method for random forest}
\usage{
\method{print}{spot2RandomForest}(x, ...)
}
\arguments{
\item{...}{not used}

\item{object}{fit of the model, an object of class \code{"spot2RandomForest"}, produced by \code{\link{buildRandomForest}}.}
}
\description{
Wrapper to \code{print.randomForest}.
}
\keyword{internal}

